/*
  Copyright (c) 2013-2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "types.hpp"

#include <memory>
#include <string>
#include <unordered_map>


namespace MMSS {

class config_t : public nocopy_t {
private:
	context_t *mmss;

	int rand_seed;

	std::shared_ptr<const protocol_t> default_proto;
	std::unordered_map<std::string, std::shared_ptr<const protocol_t>> protos;

	const std::shared_ptr<network_t> *current_net;
	std::unordered_map<std::string, std::shared_ptr<network_t>> networks;

	const std::shared_ptr<node_t> *current_node;
	std::unordered_map<std::string, std::shared_ptr<node_t>> nodes;

	config_t(context_t *mmss0) : mmss(mmss0), rand_seed(0) {}

public:
	context_t* get_context() const {
		return mmss;
	}

	bool load_proto(const char *name, const char *module, bool def);

	bool add_network(const char *name);
	bool set_packet_loss(float etx);
	bool set_packet_loss(float etx_min, float etx_max, float period, float phase);

	bool add_node(const char *name, const char *proto);
	bool add_iface(const char *name, const char *net, const gmrf_addr_t *addr);

	static std::shared_ptr<const config_t> read(context_t *mmss, const char *filename);
};

}
