/*
  Copyright (c) 2013-2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "network.hpp"

#include <memory>
#include <string>


namespace MMSS {

class iface_t : public ::gmrf_iface, public std::enable_shared_from_this<iface_t>, public nocopy_t {
private:
	node_t *node;
	network_t *net;

	std::string name;
	gmrf_addr_t address;

	gmrf_iface_state *state;

	void enqueue(context_t *mmss, const std::shared_ptr<iface_t> &dest, const void *data, size_t len);

	iface_t(node_t *node0, network_t *net0, const std::string &name0, const gmrf_addr_t *address0)
		: node(node0), net(net0), name(name0), address(*address0) {}

public:
	node_t* get_node() const {
		return node;
	}

	network_t* get_network() const {
		return net;
	}

	const std::string& get_name() const {
		return name;
	}

	const gmrf_addr_t* get_address() const {
		return &address;
	}

	gmrf_iface_state* get_state() const {
		return state;
	}

	void send(const void *data, size_t len, const gmrf_addr_t *dest) {
		net->send(data, len, this, dest);
	}

	void send_bc(const void *data, size_t len) {
		net->send_bc(data, len, this);
	}

	static std::shared_ptr<iface_t> add(const std::shared_ptr<node_t> &node, const std::shared_ptr<network_t> &net, const std::string &name, const gmrf_addr_t *address);
};

}
