/*
  Copyright (c) 2013-2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "types.hpp"
#include <config.yy.hpp>

#include <cstdio>
#include <cstring>


namespace MMSS {

class lex_t : public nocopy_t {
private:
	std::FILE *file;

	bool needspace;

	size_t start;
	size_t end;
	size_t tok_len;
	char buffer[1024];


	bool advance();
	bool next(YYLTYPE *yylloc, bool move);
	void consume(bool needspace);

	int io_error(YYSTYPE *yylval);
	int syntax_error(YYSTYPE *yylval);
	int consume_comment(YYSTYPE *yylval, YYLTYPE *yylloc);
	int unterminated_string(YYSTYPE *yylval);

	int parse_string(YYSTYPE *yylval, YYLTYPE *yylloc);
	int parse_address(YYSTYPE *yylval, YYLTYPE *yylloc);
	int parse_float(YYSTYPE *yylval, YYLTYPE *yylloc);
	int parse_number(YYSTYPE *yylval, YYLTYPE *yylloc);
	int parse_keyword(YYSTYPE *yylval, YYLTYPE *yylloc);


	char current() {
		return buffer[start + tok_len];
	}

	char* get_token() {
		return strndup(buffer+start, tok_len);
	}


public:
	lex_t(FILE *file);

	int lex(YYSTYPE *yylval, YYLTYPE *yylloc);
};

}
