/*
  Copyright (c) 2013-2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "network.hpp"
#include "context.hpp"
#include "node.hpp"

#include <cmath>
#include <cstdlib>


namespace MMSS {

float network_t::get_etx() const {
	if (etx_min == etx_max)
		return etx_min;

	float val = float(std::sin(2*M_PI * (mmss->now()/1000.0 + phase)/period));
	float diff = etx_max - etx_min;

	mmss->logf(LOG_DEBUG, "ETX for `%s' is %.3f now", name.c_str(), etx_min + diff * (val+1)/2);

	return etx_min + diff * (val+1)/2;
}

bool network_t::drop_packet() {
	int r = rand_r(&rand_seed);

	return (r > (RAND_MAX/get_etx()));
}

void network_t::enqueue(const void *data, size_t len, const iface_t *src_iface, const std::shared_ptr<iface_t> &dest_iface) {
	if (drop_packet())
		return;

	std::shared_ptr<packet_t> packet = std::make_shared<packet_t>(src_iface->get_address(), dest_iface, data, len);
	mmss->queue_event(std::move(packet), mmss->now()+1);
}

void network_t::send(const void *data, size_t len, const iface_t *src_iface, const gmrf_addr_t *dest) {
	for (auto dest_iface : interfaces) {
		if (gmrf_addr_equal(dest_iface->get_address(), dest)) {
			enqueue(data, len, src_iface, dest_iface);
			return;
		}
	}
}

void network_t::send_bc(const void *data, size_t len, const iface_t *src_iface) {
	for (auto dest_iface : interfaces) {
		if (dest_iface.get() != src_iface)
			enqueue(data, len, src_iface, dest_iface);
	}
}

}
