/*
  Copyright (c) 2013-2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "iface.hpp"
#include "protocol.hpp"

#include <memory>
#include <string>
#include <unordered_set>


namespace MMSS {

class node_t : public ::gmrf, public std::enable_shared_from_this<node_t>, public nocopy_t {
private:
	context_t *mmss;

	std::string name;

	gmrf_context_t *gmrf_ctx;
	unsigned rand_seed;
	std::shared_ptr<const protocol_t> proto;

	std::unordered_set<std::shared_ptr<iface_t>> interfaces;

	node_t(context_t *mmss0, const std::string &name0, unsigned rand_seed0, const std::shared_ptr<const protocol_t> &proto0) :
		mmss(mmss0), name(name0), gmrf_ctx(nullptr), rand_seed(rand_seed0), proto(proto0) {
	}

public:
	static std::shared_ptr<node_t> create(context_t *mmss, const std::string &name, unsigned rand_seed, const std::shared_ptr<const protocol_t> &proto) {
		auto node = std::shared_ptr<node_t>(new node_t(mmss, name, rand_seed, proto));
		node->gmrf_ctx = proto->init(node.get());
		return node;
	}

	gmrf_iface_state_t* add_iface(const std::shared_ptr<iface_t> &iface) {
		interfaces.insert(iface);
		return proto->add_iface(gmrf_ctx, iface.get());
	}

	context_t* get_context() const {
		return mmss;
	}

	const std::string& get_name() const {
		return name;
	}

	uint8_t random_byte() {
		return rand_r(&rand_seed);
	}

	void handle_packet(const std::shared_ptr<iface_t> &iface, const gmrf_addr_t *source, const void *data, size_t len) {
		proto->handle_packet(gmrf_ctx, iface->get_state(), source, data, len);
	}

	void handle_scheduled(gmrf_scheduled_func f, void *arg) {
		f(gmrf_ctx, arg);
	}
};

}
