/*
  Copyright (c) 2013-2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "now.hpp"

#include <list>
#include <memory>


namespace MMSS {

template<typename T> class timeout_queue_t : public nocopy_t {
	struct element_t {
		std::shared_ptr<T> data;
		uint64_t timeout;

		element_t(std::shared_ptr<T> &&data0, uint64_t timeout0) : data(std::move(data0)), timeout(timeout0) {}
	};

	const now_t *clock;

	std::list<element_t> queue;

public:
	timeout_queue_t(const now_t *clock0) : clock(clock0) {}

	void put(std::shared_ptr<T> &&data, uint64_t timeout) {
		auto it = queue.begin(), end = queue.end();

		while (it != end && it->timeout < timeout)
			++it;

		queue.emplace(it, std::move(data), timeout);
	}

	std::shared_ptr<T> get() {
		if (queue.empty())
			return std::shared_ptr<T>();

		element_t &el = queue.front();

		if (el.timeout > clock->now())
			return std::shared_ptr<T>();

		std::shared_ptr<T> ret(std::move(el.data));
		queue.pop_front();
		return ret;
	}

	int timeout() const {
		if (queue.empty())
			return -1;

		int diff = queue.front().timeout - clock->now();

		if (diff < 0)
			return 0;
		else
			return diff;
	}
};

}
